"use strict";

exports.__esModule = true;
exports.createRouter = createRouter;
exports.flatten = flatten;
exports.merge = merge;

var _match = require("./match");

var _history = require("./history");

var _qs = require("./qs");

var _excluded = ["path", "routes"];

function _createForOfIteratorHelperLoose(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (it) return (it = it.call(o)).next.bind(it); if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; return function () { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function createRouter(options) {
  if (options === void 0) {
    options = {};
  }

  var history = null;
  var routes = [];
  var mode = options.mode || 'history';
  var qs = options.qs || _qs.qs;
  var sync = options.sync || false;
  var router = {
    listen: function listen(routeMap, cb) {
      if (history) {
        throw new Error('Already listening');
      }

      routes = flatten(routeMap);

      var onHistoryChange = function onHistoryChange(url) {
        return transition(router, url, cb);
      };

      history = (0, _history.createHistory)(onHistoryChange, {
        mode: mode,
        sync: sync
      });
      return function () {
        history.destroy();
        history = null;
        routes = [];
      };
    },
    navigate: function navigate(to) {
      var url;
      var replace = false;

      if (to.url) {
        url = to.url;
      } else {
        url = router.href(to);
      }

      replace = !!to.replace;

      if (replace) {
        history.replace(url);
      } else {
        history.push(url);
      }
    },
    href: function href(to) {
      if (to.merge) {
        var curr = router.match(router.getUrl());
        to = merge(curr, to);
      }

      var url = to.pathname || '/';

      if (to.params) {
        Object.keys(to.params).forEach(function (param) {
          url = url.replace(':' + param, to.params[param]);
        });
      }

      if (to.query && Object.keys(to.query).length) {
        var query = qs.stringify(to.query);

        if (query) {
          url = url + '?' + query;
        }
      }

      if (to.hash) {
        var prefix = to.hash.startsWith('#') ? '' : '#';
        url = url + prefix + to.hash;
      }

      return url;
    },
    match: function match(url) {
      var route = (0, _match.match)(routes, url, qs);

      if (route) {
        return _extends({}, route, {
          data: data(routes, route)
        });
      }
    },
    getUrl: function getUrl() {
      return history.getUrl();
    }
  };
  return router;
}

function flatten(routeMap) {
  var routes = [];
  var parentData = [];

  function addLevel(level) {
    level.forEach(function (route) {
      var _route$path = route.path,
          path = _route$path === void 0 ? '' : _route$path,
          children = route.routes,
          routeData = _objectWithoutPropertiesLoose(route, _excluded);

      routes.push({
        pattern: path,
        data: parentData.concat([routeData])
      });

      if (children) {
        parentData.push(routeData);
        addLevel(children);
        parentData.pop();
      }
    });
  }

  addLevel(routeMap);
  return routes;
}

function transition(router, url, onNavigated) {
  var route = router.match(url);

  if (route) {
    for (var _iterator = _createForOfIteratorHelperLoose(route.data), _step; !(_step = _iterator()).done;) {
      var r = _step.value;

      if (r.redirect) {
        var _url = redirectUrl(router, r.redirect, route);

        return router.navigate({
          url: _url,
          replace: true
        });
      }
    }

    onNavigated && onNavigated(route);
  }
}

function redirectUrl(router, redirect, matchingRoute) {
  if (typeof redirect === 'function') {
    redirect = redirect(matchingRoute);
  }

  return redirect.url ? redirect.url : router.href(redirect);
}

function data(routes, matchingRoute) {
  for (var i = 0; i < routes.length; i++) {
    if (routes[i].pattern === matchingRoute.pattern) {
      return routes[i].data;
    }
  }
}

function merge(curr, to) {
  var pathname = to.pathname || curr.pattern;
  var params = Object.assign({}, curr.params, to.params);
  var query = to.query === null ? null : Object.assign({}, curr.query, to.query);
  var hash = to.hash === null ? null : to.hash || curr.hash || '';
  return {
    pathname: pathname,
    params: params,
    query: query,
    hash: hash
  };
}